import { Component, OnDestroy, OnInit, ChangeDetectorRef } from '@angular/core';
import { DeploymentNotificationService } from '../services/deployment-notification.service';
import { Subscription } from 'rxjs/Subscription';
import { WebSocketService } from '../services/websocket.service';
import { LayoutCommand } from './layout-command';
import { config } from '../config';

@Component({
  selector: 'app-dashboard',
  templateUrl: './dashboard.component.html',
  styleUrls: ['./dashboard.component.scss']
})
export class DashboardComponent implements OnInit, OnDestroy {
  deploying = false;
  deploymentNotificationSubscription: Subscription;
  layout: string = config.dashboard.defaultLayout;
  command_handlers = {'layout': this.layoutHandler.bind(this)};

  constructor(private deploymentNotificationService: DeploymentNotificationService,
              private webSocketService: WebSocketService,
              private changeDetectorRef: ChangeDetectorRef) {}

  ngOnInit() {
    this.webSocketService.connect();
    this.webSocketService.observeKey<LayoutCommand>('dashboard').subscribe((event) => {
      this.command_handlers[event.data.command](event.data);
      this.changeDetectorRef.detectChanges();
    });
    this.deploymentNotificationSubscription = this.deploymentNotificationService.deploying.subscribe(
      (deploying) => this.deploying = deploying
    );
  }

  layoutHandler(data: LayoutCommand) {
    if (data.layout.match('vraw-open|vraw-closed|hraw|default-open|default-closed')) {
      this.layout = data.layout;
    } else {
      console.log('Invalid webide layout "' + data.layout + '" received!');
    }
  }

  ngOnDestroy() {
    if (this.deploymentNotificationSubscription) {
      this.deploymentNotificationSubscription.unsubscribe();
    }
  }
}
